@@ -1,11 +1,11 @@
 '''Convenience functions for use in package hooks.'''
 
 # Copyright (C) 2008 - 2011 Canonical Ltd.
-# Authors: 
+# Authors:
 #   Matt Zimmerman <mdz@canonical.com>
 #   Brian Murray <brian@ubuntu.com>
 #   Martin Pitt <martin.pitt@ubuntu.com>
-# 
+#
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
 # Free Software Foundation; either version 2 of the License, or (at your
@@ -13,27 +13,43 @@
 # the full text of the license.
 
 import subprocess
-import hashlib
 import os
+import sys
+import time
+import calendar
 import datetime
 import glob
 import re
-import string
 import stat
 import base64
 import tempfile
 import shutil
+import locale
 
-import xml.dom, xml.dom.minidom
+from gi.repository import Gio, GLib
 
-from packaging_impl import impl as packaging
+from apport.packaging_impl import impl as packaging
+
+import apport
+
+try:
+    _path_key_trans = ''.maketrans('#/-_+ ', '....._')
+except AttributeError:
+    # Python 2 variant
+    import string
+    _path_key_trans = string.maketrans('#/-_+ ', '....._')
 
-_path_key_trans = string.maketrans('#/-_+ ','....._')
 def path_to_key(path):
     '''Generate a valid report key name from a file path.
-        
+
     This will replace invalid punctuation symbols with valid ones.
     '''
+    if sys.version[0] >= '3':
+        if type(path) == type(b''):
+            path = path.decode('UTF-8')
+    else:
+        if type(path) != type(b''):
+            path = path.encode('UTF-8')
     return path.translate(_path_key_trans)
 
 def attach_file_if_exists(report, path, key=None, overwrite=True):
@@ -52,13 +68,14 @@ def attach_file_if_exists(report, path, key=None, overwrite=True):
         attach_file(report, path, key, overwrite)
 
 def read_file(path):
-    '''Return the contents of the specified path. 
-        
+    '''Return the contents of the specified path.
+
     Upon error, this will deliver a text representation of the error,
     instead of failing.
     '''
     try:
-        return open(path).read().strip()
+        with open(path, 'rb') as f:
+            return f.read().strip()
     except Exception as e:
         return 'Error: ' + str(e)
 
@@ -76,10 +93,54 @@ def attach_file(report, path, key=None, overwrite=True):
 
     # Do not clobber existing keys
     if not overwrite:
-        while report.has_key(key):
+        while key in report:
             key += '_'
     report[key] = read_file(path)
 
+def attach_conffiles(report, package, conffiles=None, ui=None):
+    '''Attach information about any modified or deleted conffiles.
+
+    If conffiles is given, only this subset will be attached. If ui is given,
+    ask whether the contents of the file may be added to the report; if this is
+    denied, or there is no UI, just mark it as "modified" in the report.
+    '''
+    modified = packaging.get_modified_conffiles(package)
+
+    for path, contents in modified.items():
+        if conffiles and path not in conffiles:
+            continue
+
+        key = 'modified.conffile.' + path_to_key(path)
+        if contents == '[deleted]':
+            report[key] = contents
+            continue
+
+        if ui:
+            response = ui.yesno('It seems you have modified the contents of "%s".  Would you like to add the contents of it to your bug report?' % path)
+            if response:
+                report[key] = contents
+            else:
+                report[key] = '[modified]'
+        else:
+            report[key] = '[modified]'
+
+        mtime = datetime.datetime.fromtimestamp(os.stat(path).st_mtime)
+        report['mtime.conffile.' + path_to_key(path)] = mtime.isoformat()
+
+def attach_upstart_overrides(report, package):
+    '''Attach information about any Upstart override files'''
+
+    try:
+        files = apport.packaging.get_files(package)
+    except ValueError:
+        return
+
+    for file in files:
+        if os.path.exists(file) and file.startswith('/etc/init/'):
+            override = file.replace('.conf', '.override')
+            key = 'upstart.' + override.replace('/etc/init/', '')
+            attach_file_if_exists(report, override, key)
+
 def attach_dmesg(report):
     '''Attach information from the kernel ring buffer (dmesg).
 
@@ -87,13 +148,14 @@ def attach_dmesg(report):
     '''
     try:
         if not report.get('BootDmesg', '').strip():
-            report['BootDmesg'] = open('/var/log/dmesg').read()
+            with open('/var/log/dmesg') as f:
+                report['BootDmesg'] = f.read()
     except IOError:
         pass
     if not report.get('CurrentDmesg', '').strip():
         dmesg = command_output(['sh', '-c', 'dmesg | comm -13 --nocheck-order /var/log/dmesg -'])
         # if an initial message was truncated by the ring buffer, skip over it
-        first_newline = dmesg.find('\n[')
+        first_newline = dmesg.find(b'\n[')
         if first_newline != -1:
             dmesg = dmesg[first_newline+1:]
         report['CurrentDmesg'] = dmesg
@@ -111,7 +173,8 @@ def attach_dmi(report):
                 continue
 
             try:
-                value = open(p).read().strip()
+                with open(p) as fd:
+                    value = fd.read().strip()
             except (OSError, IOError):
                 continue
             if value:
@@ -137,21 +200,23 @@ def attach_hardware(report):
     attach_file(report, '/proc/interrupts', 'ProcInterrupts')
     attach_file(report, '/proc/cpuinfo', 'ProcCpuinfo')
     attach_file(report, '/proc/cmdline', 'ProcKernelCmdLine')
-    attach_file(report, '/proc/modules', 'ProcModules')
     attach_file(report, '/var/log/udev', 'UdevLog')
 
     if os.path.exists('/sys/bus/pci'):
         report['Lspci'] = command_output(['lspci','-vvnn'])
     report['Lsusb'] = command_output(['lsusb'])
+    report['ProcModules'] = command_output(['sort', '/proc/modules'])
     report['UdevDb'] = command_output(['udevadm', 'info', '--export-db'])
 
     # anonymize partition labels
-    report['UdevLog'] = re.sub('ID_FS_LABEL=(.*)', 'ID_FS_LABEL=<hidden>', report['UdevLog'])
-    report['UdevLog'] = re.sub('ID_FS_LABEL_ENC=(.*)', 'ID_FS_LABEL_ENC=<hidden>', report['UdevLog'])
-    report['UdevLog'] = re.sub('by-label/(.*)', 'by-label/<hidden>', report['UdevLog'])
-    report['UdevDb'] = re.sub('ID_FS_LABEL=(.*)', 'ID_FS_LABEL=<hidden>', report['UdevDb'])
-    report['UdevDb'] = re.sub('ID_FS_LABEL_ENC=(.*)', 'ID_FS_LABEL_ENC=<hidden>', report['UdevDb'])
-    report['UdevDb'] = re.sub('by-label/(.*)', 'by-label/<hidden>', report['UdevDb'])
+    l = report['UdevLog'].decode('UTF-8', errors='replace')
+    l = re.sub('ID_FS_LABEL=(.*)', 'ID_FS_LABEL=<hidden>', l)
+    l = re.sub('ID_FS_LABEL_ENC=(.*)', 'ID_FS_LABEL_ENC=<hidden>', l)
+    l = re.sub('by-label/(.*)', 'by-label/<hidden>', l)
+    l = re.sub('ID_FS_LABEL=(.*)', 'ID_FS_LABEL=<hidden>', l)
+    l = re.sub('ID_FS_LABEL_ENC=(.*)', 'ID_FS_LABEL_ENC=<hidden>', l)
+    l = re.sub('by-label/(.*)', 'by-label/<hidden>', l)
+    report['UdevLog'] = l.encode('UTF-8')
 
     attach_dmi(report)
 
@@ -192,10 +257,11 @@ def attach_alsa(report):
 
     cards = []
     if os.path.exists('/proc/asound/cards'):
-        for line in open('/proc/asound/cards'):
-            if ']:' in line:
-                fields = line.lstrip().split()
-                cards.append(int(fields[0]))
+        with open('/proc/asound/cards') as fd:
+            for line in fd:
+                if ']:' in line:
+                    fields = line.lstrip().split()
+                    cards.append(int(fields[0]))
 
     for card in cards:
         key = 'Card%d.Amixer.info' % card
@@ -216,13 +282,12 @@ def attach_alsa(report):
                     attach_file(report, path, key)
 
     report['AudioDevicesInUse'] = command_output(
-        ['fuser','-v'] + glob.glob('/dev/dsp*') 
+        ['fuser','-v'] + glob.glob('/dev/dsp*')
             + glob.glob('/dev/snd/*')
             + glob.glob('/dev/seq*') )
 
     if os.path.exists('/usr/bin/pacmd'):
-        report['PulseSinks'] = command_output(['pacmd', 'list-sinks'])
-        report['PulseSources'] = command_output(['pacmd', 'list-sources'])
+        report['PulseList'] = command_output(['pacmd', 'list'])
 
     attach_dmi(report)
     attach_dmesg(report)
@@ -243,17 +308,20 @@ def command_available(command):
             return True
     return False
 
-def command_output(command, input = None, stderr = subprocess.STDOUT):
-    '''Try to execute given command (array) and return its stdout. 
-    
+def command_output(command, input=None, stderr=subprocess.STDOUT, keep_locale=False):
+    '''Try to execute given command (array) and return its stdout.
+
     In case of failure, a textual error gets returned. This function forces
     LC_MESSAGES to C, to avoid translated output in bug reports.
     '''
     env = os.environ.copy()
-    env['LC_MESSAGES'] = 'C'
+    if not keep_locale:
+        env['LC_MESSAGES'] = 'C'
     try:
        sp = subprocess.Popen(command, stdout=subprocess.PIPE,
-                             stderr=stderr, close_fds=True, env=env)
+                             stderr=stderr, 
+                             stdin=(input and subprocess.PIPE or None),
+                             close_fds=True, env=env)
     except OSError as e:
        return 'Error: ' + str(e)
 
@@ -272,28 +340,30 @@ def _root_command_prefix():
                     stderr=subprocess.PIPE) == 0 and \
             subprocess.call(['pgrep', '-x', '-u', str(os.getuid()), 'ksmserver'],
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0:
-        prefix = ['kdesudo', '--desktop', '/usr/share/applications/apport-kde-mime.desktop', '--']
+        prefix = ['kdesudo', '--desktop', '/usr/share/applications/apport-kde-mime.desktop', 
+                  '--', 'env', '-u', 'LANGUAGE', 'LC_MESSAGES=C']
     elif os.getenv('DISPLAY') and \
             subprocess.call(['which', 'gksu'], stdout=subprocess.PIPE,
                     stderr=subprocess.PIPE) == 0 and \
             subprocess.call(['pgrep', '-x', '-u', str(os.getuid()), 'gnome-panel|gconfd-2'],
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0:
-        prefix = ['gksu', '-D', 'Apport', '--']
+        prefix = ['gksu', '-D', 'Apport', '--', 'env', '-u', 'LANGUAGE', 'LC_MESSAGES=C']
     else:
-        prefix = ['sudo']
+        prefix = ['sudo', 'LC_MESSAGES=C', 'LANGUAGE=']
 
     return prefix
 
-def root_command_output(command, input = None, stderr = subprocess.STDOUT):
-    '''Try to execute given command (array) as root and return its stdout. 
+def root_command_output(command, input=None, stderr=subprocess.STDOUT):
+    '''Try to execute given command (array) as root and return its stdout.
 
     This passes the command through gksu, kdesudo, or sudo, depending on the
     running desktop environment.
-    
+
     In case of failure, a textual error gets returned.
     '''
     assert type(command) == type([]), 'command must be a list'
-    return command_output(_root_command_prefix() + command, input, stderr)
+    return command_output(_root_command_prefix() + command, input, stderr,
+            keep_locale=True)
 
 def attach_root_command_outputs(report, command_map):
     '''Execute multiple commands as root and put their outputs into report.
@@ -322,11 +392,8 @@ def attach_root_command_outputs(report, command_map):
         script.close()
 
         # run script
-        env = os.environ.copy()
-        env['LC_MESSAGES'] = 'C'
-        env['LANGUAGE'] = ''
         sp = subprocess.Popen(_root_command_prefix() + ['/bin/sh', script_path],
-            close_fds=True, env=env)
+            close_fds=True)
         sp.wait()
 
         # now read back the individual outputs
@@ -353,22 +420,30 @@ def recent_logfile(logfile, pattern):
     '''
     lines = ''
     try:
-        for line in open(logfile):
-            if pattern.search(line):
-                lines += line
+        with open(logfile) as f:
+            for line in f:
+                if pattern.search(line):
+                    lines += line
     except IOError:
-        return []
+        return ''
     return lines
 
-def xsession_errors(pattern):
-    '''Extract messages from ~/.xsession-errors which match a regex.
-        
-    pattern should be a "re" object.
-    '''
+def xsession_errors(pattern=None):
+    '''Extract messages from ~/.xsession-errors.
 
+    By default this parses out glib-style warnings, errors, criticals etc. and
+    X window errors.  You can specify a "re" object as pattern to customize the
+    filtering.
+
+    Please note that you should avoid attaching the whole file to reports, as
+    it can, and often does, contain sensitive and private data.
+    '''
     path = os.path.expanduser('~/.xsession-errors')
     if not os.path.exists(path):
-        return None
+        return ''
+
+    if not pattern:
+        pattern = re.compile('^(\(.*:\d+\): \w+-(WARNING|CRITICAL|ERROR))|(Error: .*No Symbols named)|([^ ]+\[\d+\]: ([A-Z]+):)|([^ ]-[A-Z]+ \*\*:)|(received an X Window System error)|(^The error was \')|(^  \(Details: serial \d+ error_code)')
 
     lines = ''
     for line in open(path):
@@ -397,19 +472,20 @@ def pci_devices(*pci_classes):
 
     result = ''
     output = command_output(['lspci','-vvmmnn'])
-    for paragraph in output.split('\n\n'):
+    for paragraph in output.split(b'\n\n'):
         pci_class = None
-        pci_subclass = None
         slot = None
 
-        for line in paragraph.split('\n'):
-            key, value = line.split(':',1)
+        for line in paragraph.split(b'\n'):
+            try:
+                key, value = line.split(b':',1)
+            except ValueError:
+                continue
             value = value.strip()
             key = key.strip()
             if key == 'Class':
                 n = int(value[-5:-1],16)
                 pci_class = (n & 0xff00) >> 8
-                pci_subclass = (n & 0x00ff)
             elif key == 'Slot':
                 slot = value
 
@@ -437,45 +513,50 @@ def files_in_package(package, globpat=None):
     return result
 
 def attach_gconf(report, package):
-    '''Attach information about gconf keys set to non-default values.'''
+    '''Obsolete'''
 
-    import gconf
-    import glib
+    # keeping a no-op function for some time to not break hooks
+    pass
 
-    client = gconf.client_get_default()
+def attach_gsettings_schema(report, schema):
+    '''Attach user-modified gsttings keys of a schema.'''
+
+    cur_value = report.get('GsettingsChanges', '')
+
+    defaults = {} # schema -> key ->  value
+    env = os.environ.copy()
+    env['XDG_CONFIG_HOME'] = '/nonexisting'
+    gsettings = subprocess.Popen(['gsettings', 'list-recursively', schema],
+            env=env, stdout=subprocess.PIPE)
+    for l in gsettings.stdout:
+        try:
+            (schema, key, value) = l.split(None, 2)
+            value = value.rstrip()
+        except ValueError:
+            continue # invalid line
+        defaults.setdefault(schema, {})[key] = value
+
+    gsettings = subprocess.Popen(['gsettings', 'list-recursively', schema],
+            stdout=subprocess.PIPE)
+    for l in gsettings.stdout:
+        try:
+            (schema, key, value) = l.split(None, 2)
+            value = value.rstrip()
+        except ValueError:
+            continue # invalid line
+
+        if value != defaults.get(schema, {}).get(key, ''):
+            cur_value += '%s %s %s\n' % (schema, key, value)
+
+    report['GsettingsChanges'] = cur_value
+
+def attach_gsettings_package(report, package):
+    '''Attach user-modified gsettings keys of all schemas in a package.'''
 
-    non_defaults = {}
     for schema_file in files_in_package(package,
-                                    '/usr/share/gconf/schemas/*.schemas'):
-
-        for key, default_value in _parse_gconf_schema(schema_file).items():
-            try:
-                value = client.get(key)
-                if not value:
-                    continue
-                value = value.to_string()
-                if not default_value or value != default_value:
-                    non_defaults[key] = value
-            except glib.GError:
-                # Fall back to gconftool-2 and string comparison
-                value = command_output(['gconftool-2','-g',key])
-
-                if value != default_value:
-                    non_defaults[key] = value
-
-    if non_defaults:
-        s = ''
-        keys = non_defaults.keys()
-        keys.sort()
-        for key in keys:
-            value = non_defaults[key]
-            s += '%s=%s\n' % (key, value)
-
-        if 'GConfNonDefault' in report:
-            # This splits the lists with a newline for readability
-            s = '%s\n%s' % (report['GConfNonDefault'], s)
-
-        report['GConfNonDefault'] = s
+            '/usr/share/glib-2.0/schemas/*.gschema.xml'):
+        schema = os.path.basename(schema_file)[:-12]
+        attach_gsettings_schema(report, schema)
 
 def attach_network(report):
     '''Attach generic network-related information to report.'''
@@ -493,7 +574,10 @@ def attach_wifi(report):
     '''Attach wireless (WiFi) network information to report.'''
 
     report['WifiSyslog'] = recent_syslog(re.compile(r'(NetworkManager|modem-manager|dhclient|kernel|wpa_supplicant)(\[\d+\])?:'))
-    report['IwConfig'] = re.sub('Encryption key:(.*)', 'Encryption key: <hidden>', command_output(['iwconfig']))
+    report['IwConfig'] = re.sub('ESSID:(.*)', 'ESSID:<hidden>',
+        re.sub('Encryption key:(.*)', 'Encryption key: <hidden>',
+        re.sub('Access Point: (.*)', 'Access Point: <hidden>',
+            command_output(['iwconfig']).decode('UTF-8', errors='ignore'))))
     report['RfKill'] = command_output(['rfkill', 'list'])
     report['CRDA'] = command_output(['iw', 'reg', 'get'])
 
@@ -559,7 +643,7 @@ def attach_related_packages(report, packages):
 
 def package_versions(*packages):
     '''Return a text listing of package names and versions.
-    
+
     Arguments may be package names or globs, e. g. "foo*"
     '''
     versions = []
@@ -626,7 +710,7 @@ def _get_module_license(module):
     try:
         modinfo = subprocess.Popen(['/sbin/modinfo', module],
                 stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        out = modinfo.communicate()[0]
+        out = modinfo.communicate()[0].decode('UTF-8')
         if modinfo.returncode != 0:
             return 'invalid'
     except OSError:
@@ -644,7 +728,8 @@ def nonfree_kernel_modules(module_list = '/proc/modules'):
     '''Check loaded modules and return a list of those which are not free.'''
 
     try:
-        mods = [l.split()[0] for l in open(module_list)]
+        with open(module_list) as f:
+            mods = [l.split()[0] for l in f]
     except IOError:
         return []
 
@@ -656,36 +741,6 @@ def nonfree_kernel_modules(module_list = '/proc/modules'):
 
     return nonfree
 
-def _parse_gconf_schema(schema_file):
-    ret = {}
-
-    dom = xml.dom.minidom.parse(schema_file)
-    for gconfschemafile in dom.getElementsByTagName('gconfschemafile'):
-        for schemalist in gconfschemafile.getElementsByTagName('schemalist'):
-            for schema in schemalist.getElementsByTagName('schema'):
-                try:
-                    key = schema.getElementsByTagName('applyto')[0].childNodes[0].data
-                except IndexError:
-                    # huh, no <applyto>? let's use <key>; it has /schemas/
-                    # prefix, but it should be clear enough
-                    key = schema.getElementsByTagName('key')[0].childNodes[0].data
-                    if key.startswith('/schemas/'):
-                        key = key[8:]
-                type = schema.getElementsByTagName('type')[0].childNodes[0].data
-                try:
-                    default = schema.getElementsByTagName('default')[0].childNodes[0].data
-                    if type == 'bool':
-                        if default.lower() == 'true':
-                            ret[key] = 'true'
-                        else:
-                            ret[key] = 'false'
-                    else:
-                        ret[key] = default
-                except IndexError:
-                    ret[key] = '' # no gconf default
-
-    return ret
-
 def __drm_con_info(con):
     info = ''
     for f in os.listdir(con):
@@ -716,148 +771,49 @@ def attach_drm_info(report):
             # DRM can set an arbitrary string for its connector paths.
             report['DRM.' + path_to_key(f)] = __drm_con_info(con)
 
-#
-# Unit test
-#
+def in_session_of_problem(report):
+    '''Check if the problem happened in the currently running XDG session.
 
-if __name__ == '__main__':
+    This can be used to determine if e. g. ~/.xsession-errors is relevant and
+    should be attached.
 
-    import unittest
+    Return None if this cannot be determined due to not being able to talk to
+    ConsoleKit.
+    '''
+    # report time is in local TZ
+    orig_ctime = locale.getlocale(locale.LC_TIME)
+    try:
+        locale.setlocale(locale.LC_TIME, 'C')
+        report_time = time.mktime(time.strptime(report['Date']))
+    except KeyError:
+        return None
+    finally:
+        locale.setlocale(locale.LC_TIME, orig_ctime)
 
-    class _T(unittest.TestCase):
-        def test_module_license_evaluation(self):
-            '''module licenses can be validated correctly.'''
+    try:
+        bus = Gio.bus_get_sync(Gio.BusType.SYSTEM, None)
+        ck_manager = Gio.DBusProxy.new_sync(bus, Gio.DBusProxyFlags.NONE, None,
+            'org.freedesktop.ConsoleKit', '/org/freedesktop/ConsoleKit/Manager',
+            'org.freedesktop.ConsoleKit.Manager', None)
 
-            def _build_ko(license):
-                asm = tempfile.NamedTemporaryFile(prefix='%s-' % (license),
-                                                  suffix='.S')
-                asm.write('.section .modinfo\n.string "license=%s"\n' % (license))
-                asm.flush()
-                ko = tempfile.NamedTemporaryFile(prefix='%s-' % (license),
-                                                 suffix='.ko')
-                subprocess.call(['/usr/bin/as',asm.name,'-o',ko.name])
-                return ko
-            
-            good_ko = _build_ko('GPL')
-            bad_ko  = _build_ko('BAD')
+        cur_session = ck_manager.GetCurrentSession()
 
-            # test:
-            #  - loaded real module
-            #  - unfindable module
-            #  - fake GPL module
-            #  - fake BAD module
+        ck_session = Gio.DBusProxy.new_sync(bus, Gio.DBusProxyFlags.NONE, None,
+            'org.freedesktop.ConsoleKit', cur_session,
+            'org.freedesktop.ConsoleKit.Session', None)
 
-            # direct license check
-            self.assertTrue('GPL' in _get_module_license('isofs'))
-            self.assertEqual(_get_module_license('does-not-exist'), 'invalid')
-            self.assertTrue('GPL' in _get_module_license(good_ko.name))
-            self.assertTrue('BAD' in _get_module_license(bad_ko.name))
+        session_start_time = ck_session.GetCreationTime()
+    except GLib.GError as e:
+        sys.stderr.write('Error connecting to ConsoleKit: %s\n' % str(e))
+        return None
 
-            # check via nonfree_kernel_modules logic
-            f = tempfile.NamedTemporaryFile()
-            f.write('isofs\ndoes-not-exist\n%s\n%s\n' %
-                    (good_ko.name,bad_ko.name))
-            f.flush()
-            nonfree = nonfree_kernel_modules(f.name)
-            self.assertFalse('isofs' in nonfree)
-            self.assertTrue('does-not-exist' in nonfree)
-            self.assertFalse(good_ko.name in nonfree)
-            self.assertTrue(bad_ko.name in nonfree)
+    m = re.match('(\d{4}-\d\d-\d\dT\d\d:\d\d:\d\d)(?:\.\d+Z)$',
+            session_start_time)
+    if m:
+        # CK gives UTC time
+        session_start_time = calendar.timegm(time.strptime(m.group(1), '%Y-%m-%dT%H:%M:%S'))
+    else:
+        sys.stderr.write('cannot parse time returned by CK: %s\n' % session_start_time)
+        return None
 
-        def test_attach_dmesg(self):
-            '''attach_dmesg() does not overwrite already existing data'''
-
-            report = {}
-
-            attach_dmesg(report)
-            self.assertTrue(report['BootDmesg'].startswith('['))
-            self.assertTrue(len(report['BootDmesg']) > 500)
-            self.assertTrue(report['CurrentDmesg'].startswith('['))
-
-        def test_dmesg_overwrite(self):
-            '''attach_dmesg() does not overwrite already existing data'''
-
-            report = {'BootDmesg': 'existingboot'}
-
-            attach_dmesg(report)
-            self.assertEqual(report['BootDmesg'][:50], 'existingboot')
-            self.assertTrue(report['CurrentDmesg'].startswith('['))
-            
-            report = {'BootDmesg': 'existingboot', 'CurrentDmesg': 'existingcurrent' }
-
-            attach_dmesg(report)
-            self.assertEqual(report['BootDmesg'], 'existingboot')
-            self.assertEqual(report['CurrentDmesg'], 'existingcurrent')
-
-        def test_attach_file(self):
-            '''attach_file()'''
-
-            with open('/etc/motd') as f:
-                motd_contents = f.read().strip()
-            with open('/etc/issue') as f:
-                issue_contents = f.read().strip()
-
-            # default key name
-            report = {}
-            attach_file(report, '/etc/motd')
-            self.assertEqual(report.keys(), ['.etc.motd'])
-            self.assertEqual(report['.etc.motd'], motd_contents)
-
-            # custom key name
-            report = {}
-            attach_file(report, '/etc/motd', 'Motd')
-            self.assertEqual(report.keys(), ['Motd'])
-            self.assertEqual(report['Motd'], motd_contents)
-
-            # nonexisting file
-            report = {}
-            attach_file(report, '/nonexisting')
-            self.assertEqual(report.keys(), ['.nonexisting'])
-            self.assertTrue(report['.nonexisting'].startswith('Error: '))
-
-            # existing key
-            report = {}
-            attach_file(report, '/etc/motd')
-            attach_file(report, '/etc/motd')
-            self.assertEqual(report.keys(), ['.etc.motd'])
-            self.assertEqual(report['.etc.motd'], motd_contents)
-
-            attach_file(report, '/etc/issue', '.etc.motd', overwrite=False)
-            self.assertEqual(sorted(report.keys()), ['.etc.motd', '.etc.motd_'])
-            self.assertEqual(report['.etc.motd'], motd_contents)
-            self.assertEqual(report['.etc.motd_'], issue_contents)
-
-        def test_attach_file_if_exists(self):
-            '''attach_file_if_exists()'''
-
-            with open('/etc/motd') as f:
-                motd_contents = f.read().strip()
-
-            # default key name
-            report = {}
-            attach_file_if_exists(report, '/etc/motd')
-            self.assertEqual(report.keys(), ['.etc.motd'])
-            self.assertEqual(report['.etc.motd'], motd_contents)
-
-            # custom key name
-            report = {}
-            attach_file_if_exists(report, '/etc/motd', 'Motd')
-            self.assertEqual(report.keys(), ['Motd'])
-            self.assertEqual(report['Motd'], motd_contents)
-
-            # nonexisting file
-            report = {}
-            attach_file_if_exists(report, '/nonexisting')
-            self.assertEqual(report.keys(), [])
-
-        def test_no_crashes(self):
-            '''functions do not crash (very shallow)'''
-
-            report = {}
-            attach_hardware(report)
-            attach_alsa(report)
-            attach_network(report)
-            attach_wifi(report)
-            attach_printing(report)
-
-    unittest.main()
+    return session_start_time <= report_time
