@@ -1,8 +1,8 @@
 '''Abstraction of packaging operations.'''
 
-# Copyright (C) 2007 - 2009 Canonical Ltd.
+# Copyright (C) 2007 - 2011 Canonical Ltd.
 # Author: Martin Pitt <martin.pitt@ubuntu.com>
-# 
+#
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
 # Free Software Foundation; either version 2 of the License, or (at your
@@ -12,14 +12,14 @@
 class PackageInfo:
     def get_version(self, package):
         '''Return the installed version of a package.
-        
+
         Throw ValueError if package does not exist.
         '''
         raise NotImplementedError('this method must be implemented by a concrete subclass')
 
     def get_available_version(self, package):
         '''Return the latest available version of a package.
-        
+
         Throw ValueError if package does not exist.
         '''
         raise NotImplementedError('this method must be implemented by a concrete subclass')
@@ -31,11 +31,21 @@ class PackageInfo:
 
     def get_source(self, package):
         '''Return the source package name for a package.
-        
+
         Throw ValueError if package does not exist.
         '''
         raise NotImplementedError('this method must be implemented by a concrete subclass')
 
+    def get_package_origin(self, package):
+        '''Return package origin.
+
+        Return the repository name from which a package was installed, or None
+        if it cannot be determined.
+
+        Throw ValueError if package is not installed.
+        '''
+        raise NotImplementedError('this method must be implemented by a concrete subclass')
+
     def is_distro_package(self, package):
         '''Check package origin.
 
@@ -66,11 +76,21 @@ class PackageInfo:
 
         raise NotImplementedError('this method must be implemented by a concrete subclass')
 
+    def get_modified_conffiles(self, package):
+        '''Return modified configuration files of a package.
+
+        Return a file name -> file contents map of all configuration files of
+        package. Please note that apport.hookutils.attach_conffiles() is the
+        official user-facing API for this, which will ask for confirmation and
+        allows filtering.
+        '''
+        return {}
+
     def get_file_package(self, file, uninstalled=False, map_cachedir=None):
         '''Return the package a file belongs to.
-        
+
         Return None if the file is not shipped by any package.
-        
+
         If uninstalled is True, this will also find files of uninstalled
         packages; this is very expensive, though, and needs network access and
         lots of CPU and I/O resources. In this case, map_cachedir can be set to
@@ -81,14 +101,14 @@ class PackageInfo:
 
     def get_system_architecture(self):
         '''Return the architecture of the system.
-        
+
         This should use the notation of the particular distribution.
         '''
         raise NotImplementedError('this method must be implemented by a concrete subclass')
 
     def get_library_paths(self):
         '''Return a list of default library search paths.
-        
+
         The entries should be separated with a colon ':', like for
         $LD_LIBRARY_PATH. This needs to take any multiarch directories into
         account.
@@ -98,7 +118,7 @@ class PackageInfo:
 
     def set_mirror(self, url):
         '''Explicitly set a distribution mirror URL.
-        
+
         This might be called for operations that need to fetch distribution
         files/packages from the network.
 
@@ -109,7 +129,7 @@ class PackageInfo:
 
     def get_source_tree(self, srcpackage, dir, version=None):
         '''Download a source package and unpack it into dir..
-        
+
         dir should exist and be empty.
 
         This also has to care about applying patches etc., so that dir will
@@ -138,7 +158,7 @@ class PackageInfo:
         some init script needs to set that value based on a configuration file.
         This also determines whether Apport generates reports for Python,
         package, or kernel crashes.
-        
+
         Implementations should parse the configuration file which controls
         Apport (such as /etc/default/apport in Debian/Ubuntu).
         '''
@@ -170,12 +190,16 @@ class PackageInfo:
 
         packages is a list of ('packagename', 'version') tuples. If the version
         is None, it should install the most current available version.
-        
+
         If cache_dir is given, then the downloaded packages will be stored
         there, to speed up subsequent retraces.
 
         Return a string with outdated packages, or None if all packages were
         installed.
+
+        If something is wrong with the environment (invalid configuration,
+        package servers down, etc.), this should raise a SystemError with a
+        meaningful error message.
         '''
         raise NotImplementedError('this method must be implemented by a concrete subclass')
 
@@ -184,4 +208,20 @@ class PackageInfo:
 
         raise NotImplementedError('this method must be implemented by a concrete subclass')
 
+    def get_uninstalled_package(self):
+        '''Return a valid package name which is not installed.
+
+        This is only used in the test suite. The default implementation should
+        work, but might be slow for your backend, so you might want to
+        reimplement this.
+        '''
+        for p in self.package_name_glob('*'):
+            if not self.is_distro_package(p):
+                continue
+            try:
+                self.get_version(p)
+                continue
+            except ValueError:
+                return p
+
 import apport.packaging_impl
