@@ -1,8 +1,6 @@
 #!/usr/bin/python
 
-import os, sys
-from stat import *
-from tempfile import mkstemp
+import os, sys, stat
 
 def dump_acpi_table(filename, tablename, out):
     '''Dump a single ACPI table'''
@@ -17,7 +15,7 @@ def dump_acpi_table(filename, tablename, out):
             if (n & 15) == 0:
                 hex_str = '  %4.4x: ' % n
                 ascii_str=''
-                
+
             hex_str = hex_str + '%2.2x ' % val
 
             if (val < 32) or (val > 126):
@@ -36,15 +34,15 @@ def dump_acpi_table(filename, tablename, out):
             out.write('%s %s\n' % (hex_str, ascii_str))
         f.close()
     out.write('\n')
-    
+
 def dump_acpi_tables(path, out):
     '''Dump ACPI tables'''
 
     tables = os.listdir(path)
     for tablename in tables:
         pathname = os.path.join(path, tablename)
-        mode = os.stat(pathname)[ST_MODE]   
-        if S_ISDIR(mode):
+        mode = os.stat(pathname).st_mode
+        if stat.S_ISDIR(mode):
             dump_acpi_tables(pathname, out)
         else:
             dump_acpi_table(pathname, tablename, out)
