@@ -22,6 +22,7 @@ def add_info(report, ui):
     # check for low space
     mounts = {'/': 'system',
               '/var': '/var',
+              '/tmp': '/tmp',
              }
 
     home = os.getenv('HOME')
@@ -42,10 +43,10 @@ and installing updates. Please free some space.' % (mounts[mount], free_mb)
     if 'ExecutablePath' in report:
         path = report['ExecutablePath']
         if (apport.hookutils.links_with_shared_library(path, 'libgtk') or
-            apport.hookutils.links_with_shared_library(path, 'libX11')):
-
-            pattern = re.compile('^(\(.*:\d+\): \w+-(WARNING|CRITICAL|ERROR))|(Error: .*No Symbols named)')
-            xsession_errors = apport.hookutils.xsession_errors(pattern)
+            apport.hookutils.links_with_shared_library(path, 'libgtk-3') or
+            apport.hookutils.links_with_shared_library(path, 'libX11')) and \
+           apport.hookutils.in_session_of_problem(report):
+            xsession_errors = apport.hookutils.xsession_errors()
             if xsession_errors:
                 report['XsessionErrors'] = xsession_errors
 
@@ -68,6 +69,10 @@ Do you want to continue the report process anyway?
             report['LocalLibraries'] = ' '.join(local_libs)
             report['Tags'] = (report.get('Tags', '') + ' local-libs').strip()
 
+    # using third-party packages?
+    if '[origin:' in report.get('Package', '') or '[origin:' in report.get('Dependencies', ''):
+        report['Tags'] = (report.get('Tags', '') + ' third-party-packages').strip()
+
     # using ecryptfs?
     if os.path.exists(os.path.expanduser('~/.ecryptfs/wrapped-passphrase')):
         report['EcryptfsInUse'] = 'Yes'
